package yrpg.application.crmvisit.plugin.form;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.form.plugin.AbstractFormPlugin;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QFilter;
import kd.sdk.plugin.Plugin;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 动态表单插件
 */
public class AddressFormPlugin extends AbstractFormPlugin implements Plugin {
    /** 单据头.行政区划字段 */
    private final static String KEY_ADMINDIVISIONFIELD = "admindivisionfield";


    /**
     * 演示如何读取、设置字段值；
     *
     * @param entryRow 单据体行号
     * @param subEntryRow 子单据体行号
     * @remark
     * 设置单据体字段，需要在参数中加上行号
     */
    private void demoFieldValue(int entryRow, int subEntryRow){

        String adminId = "";    // 行政区划Id

        // 尝试取第一个国家下的第一个最明细的行政区，填写在字段上

        // 取全部国家
        List<Map<String, String>> countrys = getCountrys();
        String countryId = countrys.get(0).get("id");
        String countryName = countrys.get(0).get("name");

        // 取国家之下，第一级行政区划
        List<Map<String, String>> adminDivisions = getAdminDivisionsByParentId(countryId);
        do {
            // 取第一个行政区
            adminId = adminDivisions.get(0).get("id");
            // 继续往下取下级行政区
            adminDivisions = getAdminDivisionsByParentId(adminId);
        } while (!adminDivisions.isEmpty());

        // 赋值：填写行政区划id
        this.getModel().setValue(KEY_ADMINDIVISIONFIELD, adminId);

        // 取值
        adminId = (String) this.getModel().getValue(KEY_ADMINDIVISIONFIELD);
        // 翻译为名称
        String adminName = getAdminDivisionName(adminId);

    }

    /**
     * 获得所有国家
     *
     * @return
     */
    private static List<Map<String, String>> getCountrys() {
        ORM orm = ORM.create();
        List<DynamicObject> dynamicObjects = orm.query("bd_country", "id,name", null, "id asc");
        List<Map<String, String>> countrys = new ArrayList<Map<String, String>>();
        for (DynamicObject dynamicObject : dynamicObjects) {
            Map<String, String> map = new HashMap<String, String>();
            map.put("id", dynamicObject.get("id").toString());
            map.put("name", dynamicObject.get("name").toString());
            countrys.add(map);
        }
        return countrys;
    }

    /**
     * 获取下级行政区划
     *
     * @param parentId 上级行政区划Id
     * @return
     */
    private static List<Map<String, String>> getAdminDivisionsByParentId(String parentId) {
        ORM orm = ORM.create();
//        QFilter filter = new QFilter("parent", "=", Long.parseLong(parentId));
        QFilter[] filters = new QFilter[] {  };
        List<DynamicObject> dynamicObjects = orm.query("bd_admindivision", "id,name", filters, "id asc");
        List<Map<String, String>> provinces = new ArrayList<Map<String, String>>();
        for (DynamicObject dynamicObject : dynamicObjects) {
            Map<String, String> map = new HashMap<String, String>();
            map.put("id", dynamicObject.get("id").toString());
            map.put("name", dynamicObject.get("name").toString());
            provinces.add(map);
        }
        return provinces;
    }

    /**
     * 获取行政区划id查询行政区划全称
     */
    private static String getAdminDivisionName(String adminId) {

        String adminName = "";

        // 条件
        ORM orm = ORM.create();
        QFilter filter = new QFilter("id", "=", Long.parseLong(adminId));
        QFilter[] filters = new QFilter[] { filter };

        // 查询
        List<DynamicObject> dynamicObjects = orm.query("bd_admindivision", "id,name,fullname,country.id,country.name", filters, "id asc");

        if (!dynamicObjects.isEmpty()){
            DynamicObject dynamicObject = dynamicObjects.get(0);
            adminName = dynamicObject.getString("fullname");

        }

        return adminName;
    }
}