package yrpg.application.crmvisit.plugin.form;

import kd.bos.base.MobileBaseShowParameter;
import kd.bos.bill.AbstractMobBillPlugIn;
import kd.bos.bill.MobileBillShowParameter;
import kd.bos.bill.OperationStatus;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.datamodel.ListSelectedRow;
import kd.bos.ext.form.control.CustomControl;
import kd.bos.form.CloseCallBack;
import kd.bos.form.ShowType;
import kd.bos.form.control.Button;
import kd.bos.form.control.Control;
import kd.bos.form.control.Label;
import kd.bos.form.events.ClosedCallBackEvent;
import kd.bos.form.events.CustomEventArgs;
import kd.bos.form.events.HyperLinkClickEvent;
import kd.bos.form.events.HyperLinkClickListener;
import kd.bos.form.events.SetFilterEvent;
import kd.bos.form.events.SetFilterListener;
import kd.bos.list.BillList;
import kd.bos.list.MobileListShowParameter;
import kd.bos.list.events.ListRowClickEvent;
import kd.bos.list.events.ListRowClickListener;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.bos.servicehelper.user.UserServiceHelper;
import kd.sdk.plugin.Plugin;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateUtils;

import java.sql.Timestamp;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.EventObject;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 单据界面插件(移动端)
 */
public class VisitEditMobBillPlugin extends AbstractMobBillPlugIn implements Plugin, SetFilterListener, HyperLinkClickListener {

    //表单标识
    private static String VISIT = "yrpg_crm_visit";

    @Override
    public void initialize() {
        super.initialize();
        BillList billList = this.getView().getControl("yrpg_billlistap");
        if (billList != null) {
            billList.addSetFilterListener(this);//添加filter监听
        }
    }

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        //新建拜访，专家维护
        this.addClickListeners("yrpg_add_visit", "yrpg_expert_apply", "yrpg_expertdetails");
        BillList list = getControl("yrpg_billlistap");
        list.addListRowClickListener(new ListRowClickListener() {
            @Override
            public void listRowClick(ListRowClickEvent evt) {
                ListRowClickListener.super.listRowClick(evt);
                Long pk = (Long)evt.getCurrentListSelectedRow().getPrimaryKeyValue();
                MobileBillShowParameter showParameter = new MobileBillShowParameter();
                DynamicObject crmVisit = queryCRMVisitCopy(pk);
                if (crmVisit!=null){
                    long creatorId = crmVisit.getLong("creator.id");
                    long userId = UserServiceHelper.getCurrentUserId();
                    if (creatorId == userId){
                        showParameter.setStatus(OperationStatus.EDIT);
                    }else {
                        showParameter.setStatus(OperationStatus.VIEW);
                        showParameter.setCustomParam("mobTBMain","false");
                    }
                }
                showParameter.setPkId(pk);
                showParameter.setFormId("yrpg_crm_visit_copy_mob");
                showParameter.getOpenStyle().setShowType(ShowType.Floating);
                showParameter.setCloseCallBack(new CloseCallBack(getPluginName(), "yrpg_crm_visit_copy_mob"));
                getView().showForm(showParameter);
            }
        });
    }

    /**
     * 添加过滤条件
     */
    @Override
    public void setFilter(SetFilterEvent paramSetFilterEvent) {
        switch (((BillList) paramSetFilterEvent.getSource()).getKey()) {
            case "yrpg_billlistap"://查询出来的数据库字段
                //当前的拜访
                Timestamp timestamp = null;
                String selectData = this.getView().getPageCache().get("selectData");
                if (selectData != null) {
                    SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH-mm-ss");
                    try {
                        timestamp = new Timestamp(dateFormat.parse(selectData).getTime());
                    } catch (ParseException ex) {
                        throw new RuntimeException(ex);
                    }
                } else {
                    Calendar calendar = Calendar.getInstance();
                    calendar.set(Calendar.HOUR_OF_DAY, 0);
                    calendar.set(Calendar.MINUTE, 0);
                    calendar.set(Calendar.SECOND, 0);
                    calendar.set(Calendar.MILLISECOND, 0);
                    timestamp = new Timestamp(calendar.getTime().getTime());
                }
                QFilter nowStartDateFilter = new QFilter("yrpg_appointment_time", QCP.equals, timestamp);
                //人员权限问题
                Set<Long> leaderIdSet = this.queryUserPower();
                QFilter userFilter = new QFilter("creator", QCP.in, leaderIdSet);
                paramSetFilterEvent.addCustomQFilter(nowStartDateFilter.and(userFilter));
                break;
            default:
        }
    }

    /**
     * 人员权限问题
     * 查询：员工上下级权限关系
     * @return
     */
    private Set<Long> queryUserPower() {
        long userId = UserServiceHelper.getCurrentUserId();
        QFilter filter = new QFilter("yrpg_leader.id", QCP.equals, userId);
        DynamicObjectCollection crmUsers = QueryServiceHelper.query("yrpg_yzj_crm_user", "yrpg_subordinate_list.yrpg_subordinate.id", filter.toArray());
        Set<Long> leaderIdSet = crmUsers.stream().map(x -> x.getLong("yrpg_subordinate_list.yrpg_subordinate.id")).collect(Collectors.toSet());
        leaderIdSet.add(userId);
        return leaderIdSet;
    }

    @Override
    public void afterCreateNewData(EventObject e) {
        super.afterCreateNewData(e);
        bindTodayVisitList();
    }

    @Override
    public void afterBindData(EventObject e) {
        super.afterBindData(e);
        String status = this.getView().getFormShowParameter().getStatus().toString();
        this.getView().getPageCache().put("calendar", status);
        //日历
        long userId = UserServiceHelper.getCurrentUserId();
        Date now = new Date();
        QFilter userFilter = new QFilter("creator", QCP.equals, userId);
        QFilter startDateFilter = new QFilter("yrpg_appointment_time", QCP.large_equals, DateUtils.addDays(now, -30));
        QFilter endDateFilter = new QFilter("yrpg_appointment_time", QCP.less_equals, DateUtils.addDays(now, 30));
        DynamicObjectCollection systemCustomList = QueryServiceHelper.query(VISIT,
                "yrpg_appointment_time", new QFilter[]{userFilter, startDateFilter, endDateFilter});
        Map<String, Object> paramsMap = new HashMap<String, Object>();
        List<String> dateList = new ArrayList<>();
        for (DynamicObject dynamicObject : systemCustomList) {
            dateList.add(dynamicObject.get("yrpg_appointment_time").toString().substring(0, 10));
        }
        paramsMap.put("dateList", dateList);

        CustomControl customcontrol = this.getView().getControl("yrpg_customcontrolap");
        customcontrol.setData(paramsMap);
    }

    @Override
    public void beforeBindData(EventObject e) {
        super.beforeBindData(e);
    }

    @Override
    public void customEvent(CustomEventArgs e) {
        String calendar = this.getView().getPageCache().get("calendar");
        if (calendar != null && calendar.equals("ADDNEW")) {
            this.getView().getPageCache().remove("calendar");
            return;
        }
        // 设计器上自定义控件的标识
        String key = e.getKey();
        // 前端通过model.invoke传给后端的数据
        String args = e.getEventArgs();
        // 前端通过model.invoke定义的事件名
        String eventName = e.getEventName();
        System.out.println("自定义控件标识: " + key + ";事件名: " + eventName + ";自定义控件回传参数: " + args);
        //this.getView().showMessage("自定义控件标识: " + key + ";事件名: " + eventName + ";自定义控件回传参数: " + args);
        switch (eventName) {
            case "selectData":
                String clickDate = args.replaceAll("/", "-") + " 00-00-00";
                SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH-mm-ss");
                Timestamp timestamp = null;
                try {
                    timestamp = new Timestamp(dateFormat.parse(clickDate).getTime());
                } catch (ParseException ex) {
                    throw new RuntimeException(ex);
                }
                this.getView().getPageCache().put("selectData", clickDate);

                setListNum(timestamp);
                this.getView().updateView("yrpg_billlistap");
                break;
        }
    }

    @Override
    public void click(EventObject evt) {
        super.click(evt);
        viewExpertApply(evt);
        viewAddVisit(evt);
        viewExpertDetails(evt);
        viewBillList(evt);
    }

    private void viewBillList(EventObject evt) {
        if ("yrpg_expertdetails".equals(((Button) evt.getSource()).getKey()) == false)
            return;
        BillList billList = getControl("yrpg_billlistap");
        ListSelectedRow listSelectedRow = billList.getCurrentSelectedRowInfo();
        Long pk = (Long) listSelectedRow.getPrimaryKeyValue();
        DynamicObject crmVisit = this.queryCRMVisitCopy(pk);
        if (crmVisit != null) {
            MobileBaseShowParameter showParameter = new MobileBaseShowParameter();
            showParameter.setFormId("yrpg_crm_expert_mob");
            showParameter.getOpenStyle().setShowType(ShowType.Modal);
            showParameter.getOpenStyle().setTargetKey("yrpg_expert_info");
            showParameter.setPkId(crmVisit.getLong("yrpg_expert.id"));
            showParameter.setStatus(OperationStatus.VIEW);
            this.getView().showForm(showParameter);
        }
    }

    /**
     * 查询 新建拜访
     * @param pk
     * @return
     */
    private DynamicObject queryCRMVisitCopy(Long pk){
        QFilter filter = new QFilter("id", QCP.equals, pk);
        DynamicObject crmVisit = QueryServiceHelper.queryOne("yrpg_crm_visit_copy", "id,yrpg_expert.id,creator.id", new QFilter[]{filter});
        return crmVisit;
    }

    /**
     * 点击专家详情
     *
     * @param evt
     */
    private void viewExpertDetails(EventObject evt) {
        if ("yrpg_billlistap".equals(((Button) evt.getSource()).getKey()) == false)
            return;
        BillList billList = getControl("yrpg_billlistap");
        ListSelectedRow listSelectedRow = billList.getCurrentSelectedRowInfo();
        Long pk = (Long) listSelectedRow.getPrimaryKeyValue();
        QFilter filter = new QFilter("id", QCP.equals, pk);
        DynamicObject visitApply = QueryServiceHelper.queryOne("yrpg_crm_visit_copy_mob", "id", new QFilter[]{filter});
        if (visitApply != null) {
            MobileBillShowParameter showParameter = new MobileBillShowParameter();
            showParameter.setFormId("yrpg_crm_visit_copy_mob");
            showParameter.getOpenStyle().setShowType(ShowType.Floating);
            showParameter.setPkId(pk);
            this.getView().showForm(showParameter);
        }
    }

    /**
     * 弹专家维护
     *
     * @param evt
     */
    private void viewExpertApply(EventObject evt) {
        if ("yrpg_expert_apply".equals(((Button) evt.getSource()).getKey()) == false)
            return;
        MobileListShowParameter showParameter = new MobileListShowParameter();
        showParameter.setFormId("bos_moblist");
        showParameter.setBillFormId("yrpg_relationship_bind");
        showParameter.getOpenStyle().setShowType(ShowType.Floating);
        this.getView().showForm(showParameter);

    }

    /**
     * 弹新建拜访
     *
     * @param evt
     */
    private void viewAddVisit(EventObject evt) {
        if ("yrpg_add_visit".equals(((Button) evt.getSource()).getKey()) == false)
            return;
        MobileBillShowParameter showParameter = new MobileBillShowParameter();
        String selectData = this.getView().getPageCache().get("selectData");
        Date visitDate = null;
        if (selectData != null) {
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
            try {
                visitDate = format.parse(selectData);
            } catch (ParseException ex) {
                throw new RuntimeException(ex);
            }
        } else {
            visitDate = new Date();
        }
        showParameter.setFormId("yrpg_crm_visit_copy_mob");//布局标识
        showParameter.setCustomParam("visitDate", visitDate);
        showParameter.setCloseCallBack(new CloseCallBack(this.getPluginName(), "yrpg_crm_visit_copy_mob"));
        showParameter.getOpenStyle().setShowType(ShowType.Floating);
        this.getView().showForm(showParameter);
    }

    @Override
    public void closedCallBack(ClosedCallBackEvent closedCallBackEvent) {
        super.closedCallBack(closedCallBackEvent);
        String actionId = closedCallBackEvent.getActionId();
        if (StringUtils.equals("yrpg_crm_visit_copy_mob", actionId)) {
            this.getView().invokeOperation("refresh");
        }
    }

    private void setListNum(Timestamp timestamp) {
        Set<Long> leaderIdSet = this.queryUserPower();
        QFilter userFilter = new QFilter("creator.id", QCP.in, leaderIdSet);
        QFilter dateFilter = new QFilter("yrpg_appointment_time", QCP.equals, timestamp);
        DynamicObjectCollection todayList = QueryServiceHelper.query(VISIT, "id", new QFilter[]{userFilter, dateFilter});
        Label label = this.getView().getControl("yrpg_labelap");
        label.setText("(" + todayList.size() + ")");
    }

    private void bindVisitList(Timestamp timestamp) {
        setListNum(timestamp);
    }

    private void bindTodayVisitList() {
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        Timestamp timestamp = new Timestamp(calendar.getTime().getTime());
        bindVisitList(timestamp);
    }

    @Override
    public void hyperLinkClick(HyperLinkClickEvent e) {
        Control source = (Control) e.getSource();
    }
}