package yrpg.application.crmvisit.plugin.operate;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.entity.ExtendedDataEntity;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.AddValidatorsEventArgs;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.validate.AbstractValidator;
import kd.sdk.plugin.Plugin;

import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 专家的基础资料库操作插件
 */
public class ExpertBasicDatabaseOpPlugin extends AbstractOperationServicePlugIn implements Plugin {
    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);
        List<String> fieldKeys = e.getFieldKeys();
        fieldKeys.add("yrpg_expert_phone");
        fieldKeys.add("yrpg_id_number");
    }

    @Override
    public void onAddValidators(AddValidatorsEventArgs e) {
        super.onAddValidators(e);
        e.addValidator(new ExpertBasicDatabaseValidator());


    }

    class ExpertBasicDatabaseValidator extends AbstractValidator {

        @Override
        public void validate() {
            String opKey = this.getOperateKey();
            ExtendedDataEntity[] dataEntities = getDataEntities();
            for (ExtendedDataEntity data : dataEntities) {
                DynamicObject entity = data.getDataEntity();
                if (opKey.equals("submit")||opKey.equals("save")) {

                    String idNumber = entity.getString("yrpg_id_number").trim();
                    if (!idNumber.isEmpty()){
                        //身份证正则
                        String iDCardRegex = "^\\d{6}(19|20)\\d{2}(0[1-9]|1[0-2])(0[1-9]|[12]\\d|3[01])\\d{3}[0-9Xx]$";
                        Pattern idCard = Pattern.compile(iDCardRegex);
                        Matcher matcher = idCard.matcher(idNumber);
                        boolean matches = matcher.matches();
                        if (!matches) {
                            this.addMessage(data, "身份证号码格式不正确");
                        }
                    }

                    String expertPhone = entity.getString("yrpg_expert_phone");
                    String phoneRegex = "^1([34578])\\d{9}$";
                    Pattern idCard = Pattern.compile(phoneRegex);
                    Matcher  matcher = idCard.matcher(expertPhone);
                    boolean  matches = matcher.matches();
                    if (!matches) {
                        this.addMessage(data, "手机号格式不正确");
                    }
                }
            }
        }
    }
}